package ssw.mj;

/**
 * A <code>Token</code> represents a terminal symbol.
 * Tokens are provided by the scanner for the parser.
 * They hold additional information about the symbol.
 * 
 * @author AW
 */
public class Token {
	
	/* ----- Token codes */
	public static final int
		none       =  0,
		ident      =  1,
		number     =  2,
		charConst  =  3,
		plus       =  4,
		minus      =  5,
		times      =  6,
		slash      =  7,
		rem        =  8,
		eql        =  9,
		neq        = 10,
		gtr        = 11,
		geq        = 12,
		less       = 13,
		leq        = 14,
		and        = 15,
		or         = 16,
		assign     = 17,
		pplus      = 18,
		mminus     = 19,
		semicolon  = 20,
		comma      = 21,
		period     = 22,
		lpar       = 23,
		rpar       = 24,
		lbrack     = 25,
		rbrack     = 26,
		lbrace     = 27,
		rbrace     = 28,
		classKW    = 29,
		continueKW = 30,
		doKW       = 31,
		elseKW     = 32,
		finalKW    = 33,
		ifKW       = 34,
		newKW      = 35,
		printKW    = 36,
		readKW     = 37,
		returnKW   = 38,
		voidKW     = 39,
		whileKW    = 40,
		eof        = 41;

	/** List of printable names for all kinds of tokens.
	 *	The token codes should index correctly into this array.
	 */
	public static final String[] names =
		{"none", "identifier", "number", "character constant", "+", "-",
		 "*", "/", "%", "==", "!=", ">", ">=", "<", "<=", "&&", "||",
		 "=", "++", "--", ";", ",", ".", "(", ")", "[", "]", "{", "}",
		 "class", "continue", "do", "else", "final", "if", "new", "print",
		 "read", "return", "void", "while", "end of file" };
	
	/** Token class (none, ident, ...). */
	int kind;
	
	/** Line number of this token. */
	int line;

	/** Column number of this token. */
	int col;

	/** Value of this token (for numbers or charactor constants). */
	int val;
	
	/** String representation of this token. */
	String string;
	
	public Token (int line, int col) { this(none, line, col, 0, null); }
	
	public Token (int kind, int line, int col) { 
		this(kind, line, col, 0, null);
	}
	
	public Token (int kind, int line, int col, String string) {
		this(kind, line, col, 0, string);
	}
	
	public Token (int kind, int line, int col, int val, String string) {
		this.kind = kind;
		this.line = line; this.col = col;
		this.val = val; this.string = string;	
	}
	
	public boolean equals (Object obj) {
		if (super.equals(obj)) return true;	// it's the same object
		
		if (!(obj instanceof Token)) return false;
		
		Token other = (Token) obj;
		
		return	kind == other.kind &&
				line == other.line && col == other.col &&
				val == other.val &&
				((string == null && other.string == null) ||
				 (string != null && string.equals(other.string)));
	}
	
	public String toString () {
		StringBuffer sb = new StringBuffer();
		sb.append("line " + line + ", col " + col + ": ");
		sb.append(names[kind]);
		if (kind == ident) sb.append(" = " + string);
		else if (kind == number) sb.append(" = " + val);
		else if (kind == charConst) sb.append(" = " + (char) val);
		return sb.toString();
	}
}
