package ssw.mj.codegen;

import ssw.mj.Parser;
import ssw.mj.symtab.Obj;
import ssw.mj.symtab.Struct;
import ssw.mj.symtab.Tab;

/**
 * MicroJava Code Item.
 * An item stores the attributes of an operand during code generation.
 * 
 * @author AW
 */
public class Item {
	// codes for the item kinds
	public static final int
		Con = 0, Local = 1, Static = 2, Stack = 3, Fld = 4, Elem = 5, Meth = 6, Cond = 7;

	public int kind;      // Con, Local, Static, Stack, Fld, Elem, Meth, Cond
	public Struct type;   // item type
	public int adr;       // Con: value; Local, Static, Fld, Meth: offset;
	public Obj obj;       // Meth: method object from symbol table
	public Label tLabel,  // Cond: true jumps
	             fLabel;  // Cond: false jumps

	public Item (int x) { kind = Con; type = Tab.intType; adr = x; }

	public Item (int kind, int adr, Struct type) {
		this.kind = kind; this.adr = adr; this.type = type;
		if (kind == Cond) { tLabel = new Label(); fLabel = new Label(); }
	}

	public Item (Obj o) {
		type = o.type; adr = o.adr;
		switch (o.kind) {
			case Obj.Con: kind = Con; break;
			case Obj.Var:
				if (o.level == 0) kind = Static; else kind = Local;
				break;
			case Obj.Meth: kind = Meth; obj = o; break;
			default: Parser.Errors.semError("cannot create Item");
		}
	}
}
