package ssw.mj.test;

import java.io.ByteArrayOutputStream;

import junit.framework.Assert;

/**
 * @author AW
 */
public class TestByteArrayOutputStream extends ByteArrayOutputStream {
	private byte[] actual, expected;
	private int curA, curE;

	TestByteArrayOutputStream () {
		actual = new byte[8192];
		expected = new byte[8192];
		curA = curE = 0;
		setCodeHeader();
	}

	public static byte[] int2ByteArray (int x) {
		byte[] b = new byte[4];
		for (int i = 3; i >= 0; i--) { b[i] = (byte) x; x = x >>> 8; }
		return b;
	}

	private void setBytes (int val, int off) {
		byte[] b = int2ByteArray(val);
		expected[off] = b[0];
		expected[off+1] = b[1];
		expected[off+2] = b[2];
		expected[off+3] = b[3];
	}
				
	void setCodeHeader () {
		expected[0] = (byte)0x4d;	// M
		expected[1] = (byte)0x4a;	// J
		setCodeSize(0);
		setDataSize(0);
		setMainPC(-1);
		curE = 14;
	}
	void setCodeSize (int x) { setBytes(x, 2); }
	void setDataSize (int x) { setBytes(x, 6); }
	void setMainPC (int x) { setBytes(x, 10); }
	
	public void write (byte b) { actual[curA++] = b; }
	
	public void write (byte[] b, int off, int len) {
		for (int i = off; i < off+len; i++)
			write(b[i]);	
	}
		
	void expect (int x, int nBytes) {
		byte[] b = int2ByteArray(x);
		switch (nBytes) {
			case 4: expected[curE++] = b[0];
			case 3: expected[curE++] = b[1];
			case 2: expected[curE++] = b[2];
			case 1:	expected[curE++] = b[3];
		}
	}
	
	public void verify () {
		if (curA != curE)
			Assert.fail("Expected " + curE + " bytes, but encountered " + curA);
		// compare bytes
		for (int i = 0; i < curA; i++)
			Assert.assertEquals("Byte " + i, expected[i], actual[i]);	
	}
	
	void dump () {
		System.out.print("Byte #          ");
		for (int i = 0; i < Math.max(curA, curE); i++)
			System.out.print(" " + int2Str(i, 4));
		System.out.println();
		System.out.print("Actual   (" + int2Str(curA, 4) + "):");
		for (int i = 0; i < curA; i++)
			System.out.print(" " + int2Str(actual[i], 4));
		System.out.println();
		System.out.print("Expected (" + int2Str(curE, 4) + "):");
		for (int i = 0; i < curE; i++)
			System.out.print(" " + int2Str(expected[i], 4));
		System.out.println();
	}
	
	static String int2Str (int x, int width) {
		assert width >= 0;
		StringBuffer sb = new StringBuffer(width);
		boolean neg = false;
		if (x < 0) { neg = true; x = -x; }
		do { sb.insert(0, x%10); x /= 10; width--; } 
		while (x > 0);
		if (neg) { sb.insert(0, '-'); width--; }
		while (width > 0) { sb.insert(0, ' '); width--; }
		return sb.toString();	
	}
}
