package ssw.mj.test;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import junit.framework.Assert;

/**
 * Allows testing of output.
 * 
 * @author AW
 */
public class TestPrintWriter extends PrintWriter {
	private List actualLines;
	private List expectedLines;
	
	private StringBuffer curLine;
	
	TestPrintWriter () { 
		super(System.out, true);
		actualLines = new ArrayList();
		expectedLines = new ArrayList();
		curLine = new StringBuffer();
	}
	
	public void print (String s) { super.print(s); curLine.append(s); }
	
	public void println (String s) {
		// print the string
		print(s);
		super.println();
		
		// start a new line
		actualLines.add(curLine.toString());
		curLine = new StringBuffer();
	}

	void addExpectedLine (String line) { expectedLines.add(line); }
	
	void verify () {
		// check number of lines
		if (actualLines.size() != expectedLines.size())
			Assert.fail("Expected " + expectedLines.size() + 
						" output lines, but encountered " +
						actualLines.size() + "\n" + printActualLines());
		// compare lines
		for (int i = 0; i < actualLines.size(); i++) {
			String expectedLine = (String) expectedLines.get(i);
			String actualLine = (String) actualLines.get(i);
			Assert.assertEquals(expectedLine, actualLine);	
		}
	}
	
	private String printActualLines () {
		StringBuffer sb = new StringBuffer();
		
		Iterator iter = actualLines.iterator();
		while (iter.hasNext())
			sb.append((String) iter.next() + "\n");
		
		return sb.toString();
	}
}
