package ssw.mj.test;

import junit.framework.TestCase;

import ssw.mj.Token;

/**
 * Test cases for <code>Token</code> class.
 * 
 * @author AW
 */
public class TokenTest extends TestCase {

	private static final int
		none       = Token.none,
		ident      = Token.ident,
		number     = Token.number,
		charConst  = Token.charConst,
		plus       = Token.plus,
		minus      = Token.minus,
		times      = Token.times,
		slash      = Token.slash,
		rem        = Token.rem,
		eql        = Token.eql,
		neq        = Token.neq,
		gtr        = Token.gtr,
		geq        = Token.geq,
		less       = Token.less,
		leq        = Token.leq,
		and        = Token.and,
		or         = Token.or,
		assign     = Token.assign,
		pplus      = Token.pplus,
		mminus     = Token.mminus,
		semicolon  = Token.semicolon,
		comma      = Token.comma,
		period     = Token.period,
		lpar       = Token.lpar,
		rpar       = Token.rpar,
		lbrack     = Token.lbrack,
		rbrack     = Token.rbrack,
		lbrace     = Token.lbrace,
		rbrace     = Token.rbrace,
		classKW    = Token.classKW,
		continueKW = Token.continueKW,
		doKW       = Token.doKW,
		elseKW     = Token.elseKW,
		finalKW    = Token.finalKW,
		ifKW       = Token.ifKW,
		newKW      = Token.newKW,
		printKW    = Token.printKW,
		readKW     = Token.readKW,
		returnKW   = Token.returnKW,
		voidKW     = Token.voidKW,
		whileKW    = Token.whileKW,
		eof        = Token.eof;

	public void testEquals () {
		// equals has to be commutative 
		// so always check both ways, i.e. t1.equals(t2) AND t2.equals(t1)
		Token t1 = new Token(ifKW, 12, 25, 0, "if");
		assertSame("Same object", t1, t1);
		assertEquals("Same object", t1, t1);
		Token t2 = new Token(ifKW, 12, 25, 0, "if");
		assertEquals("Same values (1)", t1, t2);
		assertEquals("Same values (2)", t2, t1);
		t2 = new Token(elseKW, 12, 25, 0, "if");
		assertFalse("Different kind (1)", t1.equals(t2));
		assertFalse("Different kind (2)", t2.equals(t1));
		t2 = new Token(ifKW, 14, 25, 0, "if");
		assertFalse("Different line (1)", t1.equals(t2));
		assertFalse("Different line (2)", t2.equals(t1));
		t2 = new Token(ifKW, 12, 2, 0, "if");
		assertFalse("Different column (1)", t1.equals(t2));
		assertFalse("Different column (2)", t2.equals(t1));
		t2 = new Token(ifKW, 12, 25, 1, "if");
		assertFalse("Different value (1)", t1.equals(t2));
		assertFalse("Different value (2)", t2.equals(t1));
		t2 = new Token(ifKW, 12, 25, 0, "bla");
		assertFalse("Different string (1)", t1.equals(t2));
		assertFalse("Different string (2)", t2.equals(t1));
		t1 = new Token(plus, 20, 222, 0, null);
		t2 = new Token(plus, 20, 222, 0, null);
		assertEquals("Both null strings (1)", t1, t2);
		assertEquals("Both null strings (2)", t2, t1);
		t2 = new Token(plus, 20, 222, 0, "");
		assertFalse("Only one null string (1)", t1.equals(t2));
		assertFalse("Only one null string (2)", t2.equals(t1));
	}
	
	public void testToString () {
		Token t;
		// test all token kinds
		t = new Token(none, 0, 1);
		assertEquals("String for none-token", "line 0, col 1: none", t.toString());
		t = new Token(ident, 1, 1, "aName");
		assertEquals("String for ident-token", "line 1, col 1: identifier = aName", t.toString());
		t = new Token(number, 2, 1, 1234, null);
		assertEquals("String for number-token", "line 2, col 1: number = 1234", t.toString());
		t = new Token(charConst, 3, 1, 36, null);
		assertEquals("String for chrCon-token", "line 3, col 1: character constant = $", t.toString());
		t = new Token(plus, 4, 1);
		assertEquals("String for plus-token", "line 4, col 1: +", t.toString());
		t = new Token(minus, 5, 1);
		assertEquals("String for minus-token", "line 5, col 1: -", t.toString());
		t = new Token(times, 6, 1);
		assertEquals("String for times-token", "line 6, col 1: *", t.toString());
		t = new Token(slash, 7, 1);
		assertEquals("String for slash-token", "line 7, col 1: /", t.toString());
		t = new Token(rem, 8, 1);
		assertEquals("String for rem-token", "line 8, col 1: %", t.toString());
		t = new Token(eql, 9, 1);
		assertEquals("String for eql-token", "line 9, col 1: ==", t.toString());
		t = new Token(neq, 10, 1);
		assertEquals("String for neq-token", "line 10, col 1: !=", t.toString());
		t = new Token(gtr, 11, 1);
		assertEquals("String for gtr-token", "line 11, col 1: >", t.toString());
		t = new Token(geq, 12, 1);
		assertEquals("String for geq-token", "line 12, col 1: >=", t.toString());
		t = new Token(less, 13, 1);
		assertEquals("String for less-token", "line 13, col 1: <", t.toString());
		t = new Token(leq, 14, 1);
		assertEquals("String for leq-token", "line 14, col 1: <=", t.toString());
		t = new Token(and, 15, 1);
		assertEquals("String for and-token", "line 15, col 1: &&", t.toString());
		t = new Token(or, 16, 1);
		assertEquals("String for or-token", "line 16, col 1: ||", t.toString());
		t = new Token(assign, 17, 1);
		assertEquals("String for assign-token", "line 17, col 1: =", t.toString());
		t = new Token(pplus, 18, 1);
		assertEquals("String for pplus-token", "line 18, col 1: ++", t.toString());
		t = new Token(mminus, 19, 1);
		assertEquals("String for mminus-token", "line 19, col 1: --", t.toString());
		t = new Token(semicolon, 20, 1);
		assertEquals("String for semicolon-token", "line 20, col 1: ;", t.toString());
		t = new Token(comma, 21, 1);
		assertEquals("String for comma-token", "line 21, col 1: ,", t.toString());
		t = new Token(period, 22, 1);
		assertEquals("String for period-token", "line 22, col 1: .", t.toString());
		t = new Token(lpar, 23, 1);
		assertEquals("String for lpar-token", "line 23, col 1: (", t.toString());
		t = new Token(rpar, 24, 1);
		assertEquals("String for rpar-token", "line 24, col 1: )", t.toString());
		t = new Token(lbrack, 25, 1);
		assertEquals("String for lbrack-token", "line 25, col 1: [", t.toString());
		t = new Token(rbrack, 26, 1);
		assertEquals("String for rbrack-token", "line 26, col 1: ]", t.toString());
		t = new Token(lbrace, 27, 1);
		assertEquals("String for lbrace-token", "line 27, col 1: {", t.toString());
		t = new Token(rbrace, 28, 1);
		assertEquals("String for rbrace-token", "line 28, col 1: }", t.toString());
		t = new Token(classKW, 29, 1, "class");
		assertEquals("String for classKW-token", "line 29, col 1: class", t.toString());
		t = new Token(continueKW, 30, 1, "continue");
		assertEquals("String for continueKW-token", "line 30, col 1: continue", t.toString());
		t = new Token(doKW, 31, 1, "do");
		assertEquals("String for doKW-token", "line 31, col 1: do", t.toString());
		t = new Token(elseKW, 32, 1, "else");
		assertEquals("String for elseKW-token", "line 32, col 1: else", t.toString());
		t = new Token(finalKW, 33, 1, "final");
		assertEquals("String for finalKW-token", "line 33, col 1: final", t.toString());
		t = new Token(ifKW, 34, 1, "if");
		assertEquals("String for ifKW-token", "line 34, col 1: if", t.toString());
		t = new Token(newKW, 35, 1, "new");
		assertEquals("String for newKW-token", "line 35, col 1: new", t.toString());
		t = new Token(printKW, 36, 1, "print");
		assertEquals("String for printKW-token", "line 36, col 1: print", t.toString());
		t = new Token(readKW, 37, 1, "read");
		assertEquals("String for read-token", "line 37, col 1: read", t.toString());
		t = new Token(returnKW, 38, 1, "return");
		assertEquals("String for return-token", "line 38, col 1: return", t.toString());
		t = new Token(voidKW, 39, 1, "void");
		assertEquals("String for voidKW-token", "line 39, col 1: void", t.toString());
		t = new Token(whileKW, 40, 1, "while");
		assertEquals("String for whileKW-token", "line 40, col 1: while", t.toString());
		t = new Token(eof, 41, 1);
		assertEquals("String for eof-token", "line 41, col 1: end of file", t.toString());
	}
	
	public static void main (String[] args) {
		junit.textui.TestRunner.run(TokenTest.class);	
	}
}
